import { TypeRevivers, TypeReviverMap } from "type-reviver";
export declare type StringifyReviver = (this: any, key: string, value: any, type: string, callCount: number, stringifyOptions: StringifyReviverOptions) => any;
export declare type ParseReviver = (this: any, key: string, value: any, type: string, callCount: number) => any;
export declare type SPReviver = (this: any, key: string, value: any, type: string, callCount: number, stringifyOptions: StringifyReviverOptions | undefined) => any;
export declare type Reviver = SPReviver | StringifyReviver | ParseReviver | ReviverPair;
export interface ReviverPair {
    string: StringifyReviver;
    parse: ParseReviver;
}
export declare function isReviverPair(target: any): target is ReviverPair;
export interface StringifyReviverOptions {
    skip?: boolean;
    skipMark?: boolean;
    skipRootMark?: boolean;
}
export interface JSONStringifyOptions extends StringifyReviverOptions {
    skipRoot?: boolean;
    space?: string | number;
    mark?: string | null;
}
export interface PresetTypeReviverMap {
    presetTypeReviverMap: TypeReviverMap<Reviver>;
}
export interface CustomJSONStringify extends PresetTypeReviverMap {
    <Revr extends Reviver>(value: any, typeRevivers?: TypeRevivers<Revr> | null, options?: JSONStringifyOptions): string;
}
export declare function createCustomJSONStringify(presetTypeReviverMap?: TypeReviverMap<Reviver>): CustomJSONStringify;
export interface CustomJSONParse extends PresetTypeReviverMap {
    <Revr extends Reviver>(text: string, typeRevivers?: TypeRevivers<Revr> | null, options?: JSONParseOptions): any;
}
export declare enum LostRevier {
    original = "original",
    parse = "parse",
    ignore = "ignore"
}
export interface JSONParseOptions {
    mark?: string | null;
    lostRevier?: LostRevier;
}
export declare function createCustomJSONParse(presetTypeReviverMap?: TypeReviverMap<Reviver>): CustomJSONParse;
export interface CustomJSON {
    stringify: CustomJSONStringify;
    parse: CustomJSONParse;
}
export declare function createCustomJSON(presetTypeReviverMap?: TypeReviverMap<Reviver>): CustomJSON;
/**
 * 自定义JSON序列化；可根据数据类型来自定义序列化方案； 给合  customJSONParse  可实现对任意类型的数据 进行序列化 并完整（无信息丢失）还原；
 * @param value: any   被序列化的对象
 * @param typeRevivers?:TypeRevivers|null    定义 类型 与 其对类的 自定义序列化函数；
 * @param options:JSONStringifyOptions    选项
 */
export declare const customJSONStringify: CustomJSONStringify;
/**
 * 自定义JSON解析；可根据类型自定义解析逻辑；
 * @param text
 * @param typeRevivers
 * @param options
 */
export declare const customJSONParse: CustomJSONParse;
