
/*
json-tls	v1.0.3
author: 郭斌勇
license: ISC
homepage: https://github.com/GuoBinyong/json-tls#readme
repository: https://github.com/GuoBinyong/json-tls
description: json-tls 提供了JSON相关的工具，如:customJSONStringify、customJSONParse 等
*/

import { getExactTypeNameOf, isBaseType } from 'type-tls';
import { mergeTypeRevivers, flatParseTypeRevivers, toTypeReviverArray } from 'type-reviver';
import { typeReviverArray } from 'type-reviver-json';

//ReviverPair 的类型守卫
function isReviverPair(target) {
    return target && typeof target.string === "function" && typeof target.parse === "function";
}
/**
 * 默认的 Mark 值
 */
const _defaultMark = "__MarKOfCustomJSON__";
function createCustomJSONStringify(presetTypeReviverMap) {
    /**
     * 自定义JSON序列化；可根据数据类型来自定义序列化方案； 给合  customJSONParse  可实现对任意类型的数据 进行序列化 并完整（无信息丢失）还原；
     * @param value: any   被序列化的对象
     * @param typeRevivers?:TypeRevivers|null    定义 类型 与 其对类的 自定义序列化函数；
     * @param options:JSONStringifyOptions    选项
     */
    function customJSONStringify(value, typeRevivers, options = {}) {
        try {
            let presetTRMap = customJSONStringify.presetTypeReviverMap;
            if (typeRevivers || presetTRMap.size > 0) {
                let mergedTRArr = mergeTypeRevivers(presetTRMap, typeRevivers);
                var parseInfo = flatParseTypeRevivers(mergedTRArr);
                let disDefaultArr = parseInfo.typeFun;
                //禁用toJSON
                if (disDefaultArr.length > 0) {
                    var disTypeInfos = disDefaultArr.reduce(function (infoArr, fun) {
                        let toJSON = fun.prototype.toJSON;
                        if (toJSON) {
                            fun.prototype.toJSON = undefined;
                            infoArr.push({
                                type: fun,
                                toJSON: toJSON
                            });
                        }
                        return infoArr;
                    }, []);
                }
                var trObj = parseInfo.trObject;
                if (trObj) {
                    let opts = Object.assign({}, options);
                    let mark = opts.mark;
                    mark = opts.mark = mark == null ? _defaultMark : mark;
                    let markType = mark + "Type";
                    let markValue = mark + "Value";
                    let callCount = 0; // stringifyReviver 的调用次数
                    let stringifyReviver = function (key, value) {
                        ++callCount;
                        //此处的写法没有问题，这样写是为了提高性能
                        switch (key) {
                            case mark: ;
                            case markType: ;
                            case markValue: return value;
                        }
                        if (opts.skipRoot && callCount === 1) { //需要跳过这次处理
                            return value;
                        }
                        let typeName = getExactTypeNameOf(value);
                        let revier = trObj[typeName];
                        let revierFun = (revier && typeof revier === "object") ? revier.string : revier;
                        if (!revierFun) {
                            return value;
                        }
                        let rerOpts = Object.assign({}, opts);
                        let rerRes = revierFun.call(this, key, value, typeName, callCount, rerOpts);
                        if (rerOpts.skip) { //需要放在上一句 `revier.call(this,key,value,typeName,callCount,rerOpts)` 的后面，因为 revier 可修改 rerOpts 的值
                            return value;
                        }
                        /*
                        在以下任一情况下，均不会添加 mark
                        - revier 返回 undefined  :  `rerRes === undefined`
                        - skipMark 为 true : `rerOpts.skipMark`
                        - value 是被 customJSONStringify 最初序列化的目标（即：根） 且  skipRootMark 为 true : `rerOpts.skipRootMark && callCount === 1`
                        */
                        if (rerRes === undefined || rerOpts.skipMark || (rerOpts.skipRootMark && callCount === 1)) {
                            return rerRes;
                        }
                        return {
                            [mark]: true,
                            [markType]: typeName,
                            [markValue]: rerRes
                        };
                    };
                    var jsonStr = JSON.stringify(value, stringifyReviver, opts.space);
                }
            }
            // @ts-ignore
            if (!trObj) {
                jsonStr = JSON.stringify(value, null, options.space);
            }
        }
        catch (e) {
            throw e;
        }
        finally {
            //取消禁用toJSON
            if (disTypeInfos && disTypeInfos.length > 0) {
                disTypeInfos.forEach(function (funInfo) {
                    funInfo.type.prototype.toJSON = funInfo.toJSON;
                });
            }
            // @ts-ignore
            return jsonStr;
        }
    }
    Object.defineProperty(customJSONStringify, "presetTypeReviverMap", {
        configurable: true,
        enumerable: true,
        get: function () {
            if (!this._presetTypeReviverMap) {
                this._presetTypeReviverMap = new Map();
            }
            return this._presetTypeReviverMap;
        },
        set: function (newValue) {
            if (newValue instanceof Map) {
                this._presetTypeReviverMap = newValue;
            }
        }
    });
    if (presetTypeReviverMap) {
        customJSONStringify.presetTypeReviverMap = presetTypeReviverMap;
    }
    return customJSONStringify;
}
var LostRevier;
(function (LostRevier) {
    LostRevier["original"] = "original";
    LostRevier["parse"] = "parse";
    LostRevier["ignore"] = "ignore";
})(LostRevier || (LostRevier = {}));
function createCustomJSONParse(presetTypeReviverMap) {
    /**
     * 自定义JSON解析；可根据类型自定义解析逻辑；
     * @param text
     * @param typeRevivers
     * @param options
     */
    function customJSONParse(text, typeRevivers, options = {}) {
        let lostRevier = options.lostRevier || LostRevier.parse;
        let presetTRMap = customJSONParse.presetTypeReviverMap;
        if (typeRevivers || presetTRMap.size > 0) {
            let mergedTRArr = mergeTypeRevivers(presetTRMap, typeRevivers);
            let parseInfo = flatParseTypeRevivers(mergedTRArr);
            var trObj = parseInfo.trObject;
        }
        // @ts-ignore
        if (!trObj && lostRevier === LostRevier.original) {
            return JSON.parse(text);
        }
        let mark = options.mark == null ? _defaultMark : options.mark;
        let markType = mark + "Type";
        let markValue = mark + "Value";
        let callCount = 0; // parseReviver 的调用次数
        function parseReviver(key, value) {
            ++callCount;
            if (isBaseType(value)) {
                return value;
            }
            let typeName = value[markType];
            if (!(typeName && value[mark])) {
                return value;
            }
            var realValue = value[markValue];
            let revier = trObj[typeName];
            let revierFun = (revier && typeof revier === "object") ? revier.parse : revier;
            if (!revierFun) {
                switch (lostRevier) {
                    case LostRevier.ignore: return undefined;
                    case LostRevier.original: return value;
                    default: return realValue;
                }
            }
            return revierFun.call(this, key, realValue, typeName, callCount);
        }
        return JSON.parse(text, parseReviver);
    }
    Object.defineProperty(customJSONParse, "presetTypeReviverMap", {
        configurable: true,
        enumerable: true,
        get: function () {
            if (!this._presetTypeReviverMap) {
                this._presetTypeReviverMap = new Map();
            }
            return this._presetTypeReviverMap;
        },
        set: function (newValue) {
            if (newValue instanceof Map) {
                this._presetTypeReviverMap = newValue;
            }
        }
    });
    if (presetTypeReviverMap) {
        customJSONParse.presetTypeReviverMap = presetTypeReviverMap;
    }
    return customJSONParse;
}
function createCustomJSON(presetTypeReviverMap) {
    return {
        stringify: createCustomJSONStringify(presetTypeReviverMap),
        parse: createCustomJSONParse(presetTypeReviverMap)
    };
}
/**
 * 默认预置的TypeReviverMap
 */
const defaultPresetTypeReviverMap = new Map(typeReviverArray);
/**
 * 自定义JSON序列化；可根据数据类型来自定义序列化方案； 给合  customJSONParse  可实现对任意类型的数据 进行序列化 并完整（无信息丢失）还原；
 * @param value: any   被序列化的对象
 * @param typeRevivers?:TypeRevivers|null    定义 类型 与 其对类的 自定义序列化函数；
 * @param options:JSONStringifyOptions    选项
 */
const customJSONStringify = createCustomJSONStringify(defaultPresetTypeReviverMap);
/**
 * 自定义JSON解析；可根据类型自定义解析逻辑；
 * @param text
 * @param typeRevivers
 * @param options
 */
const customJSONParse = createCustomJSONParse(defaultPresetTypeReviverMap);

/**
 * StringParseTypeRevivers 的类型守卫
 * @param target
 */
function isTypeReviversPair(target) {
    return target && typeof target === "object" && !Array.isArray(target) && (target.string || target.parse);
}
/**
 * 创建纯净的 customJSON
 */
const customJSON = createCustomJSON();
/**
 * 创建带 presetTypeReviverMap 的 deepCopyByJSON 的工厂函数
 * @param presetTypeReviverMap
 */
function createDeepCopyByJSONWith(presetTypeReviverMap) {
    /**
     * 通过 自定义 JSON 序列人的方式 对 value 进行 深拷贝
     * @param value
     * @param dcTypeRevivers
     */
    function deepCopyByJSON(value, typeReviversOpts) {
        if (isBaseType(value)) {
            return value;
        }
        let preTRArr = toTypeReviverArray(deepCopyByJSON.presetTypeReviverMap);
        let strAllTRArr = preTRArr;
        let parseAllTRArr = preTRArr;
        if (typeReviversOpts) {
            if (isTypeReviversPair(typeReviversOpts)) {
                let strTRs = typeReviversOpts.string;
                if (strTRs) {
                    let strTRArr = toTypeReviverArray(strTRs);
                    strAllTRArr = strAllTRArr.concat(strTRArr);
                }
                let parseTRs = typeReviversOpts.parse;
                if (parseTRs) {
                    let parseTRArr = toTypeReviverArray(parseTRs);
                    parseAllTRArr = parseAllTRArr.concat(parseTRArr);
                }
            }
            else {
                let trArr = toTypeReviverArray(typeReviversOpts);
                strAllTRArr = strAllTRArr.concat(trArr);
                parseAllTRArr = strAllTRArr;
            }
        }
        var str = customJSON.stringify(value, strAllTRArr);
        return customJSON.parse(str, parseAllTRArr);
    }
    Object.defineProperty(deepCopyByJSON, "presetTypeReviverMap", {
        configurable: true,
        enumerable: true,
        get: function () {
            if (!this._presetTypeReviverMap) {
                this._presetTypeReviverMap = new Map();
            }
            return this._presetTypeReviverMap;
        },
        set: function (newValue) {
            if (newValue instanceof Map) {
                this._presetTypeReviverMap = newValue;
            }
        }
    });
    if (presetTypeReviverMap) {
        deepCopyByJSON.presetTypeReviverMap = presetTypeReviverMap;
    }
    return deepCopyByJSON;
}
/**
 * 通过 自定义 JSON 序列人的方式 对 value 进行 深拷贝
 */
const deepCopyByJSON = createDeepCopyByJSONWith(new Map(typeReviverArray));

export { LostRevier, createCustomJSON, createCustomJSONParse, createCustomJSONStringify, createDeepCopyByJSONWith, customJSONParse, customJSONStringify, deepCopyByJSON, isReviverPair, isTypeReviversPair };
